import Mathutils
import bpy

from wg_utils import (
    lazy_deselect_objects, save_excursion,
    possibly_delete_object_by_id)


class Error(Exception): pass
class NoStartEndTime(Error): pass


NUKE_OPERATE_LAYERS = [
    True, False, True, True, False,
    False, True, False, False, False,
    False, False, False, False, False,
    False, False, False, False, False]
NUKE_MACHINERY_LAYERS = [False, False, False, False, True] + [False] * 15
NUKE_EXPLOSION_LAYERS = [False, False, False, True] + [False] * 16


@save_excursion
def nukify_curve(curve):
    # The curve we're nukifying MUST have a start and end time set!
    try:
        start_frame = curve['start_frame']
        flight_time = curve['flight_time']
        end_frame = start_frame + flight_time
    except KeyError:
        raise NoStartEndTime(
            'Both start_frame and end_frame must be set on a nukified curve')

    # We need to make sure that we have only the first layer visible
    # for this... that's how we work
    bpy.context.scene.visible_layers = NUKE_OPERATE_LAYERS
    bpy.context.scene.cursor_location = [0, 0, 0]

    # Delete old instances of curve stuff
    def possibly_delete_obj_by_prop(propname):
        if propname in curve:
            objname = curve[propname]

            # only delete if that object also has a reference to this
            # curve
            if (objname in bpy.context.scene.objects
                and bpy.context.scene.objects[objname].get(
                    'parent_curve') == curve.name):
                possibly_delete_object_by_id(curve[propname])
        
    possibly_delete_obj_by_prop('rocket_bevel')
    possibly_delete_obj_by_prop('rocket_taper')
    possibly_delete_obj_by_prop('rocket_hook')
    possibly_delete_obj_by_prop('explosion_circle')

    # Set the bevel object to a cloned curvecircle, animate it
    lazy_deselect_objects()
    base_bevel = bpy.data.objects['nuke_curvecircle']
    bpy.context.scene.objects.active = base_bevel
    base_bevel.selected = True
    bpy.ops.object.duplicate()
    new_bevel = bpy.context.scene.objects.active
    curve.data.bevel_object = new_bevel
    curve['rocket_bevel'] = new_bevel.name
    new_bevel['parent_curve'] = curve.name

    bpy.ops.anim.change_frame(frame=start_frame)
    bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)
    bpy.ops.anim.change_frame(frame=start_frame - 1)
    bpy.ops.transform.resize(value=(0.0, 0.0, 0.0))
    bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)

    # Duplicate/create taper object
    lazy_deselect_objects()
    base_ctrlcurve = bpy.data.objects['base_ctrlcurve']
    bpy.context.scene.objects.active = base_ctrlcurve
    base_ctrlcurve.selected = True
    bpy.ops.object.duplicate()
    new_taper_curve = bpy.context.scene.objects.active
    new_taper_curve.location = [0, 0, 0]
    curve.data.taper_object = new_taper_curve
    curve['rocket_taper'] = new_taper_curve.name
    new_taper_curve['parent_curve'] = curve.name

    # # Hook setup
    
    ## Enter edit mode
    ## Add hook to the taper object
    bpy.ops.object.mode_set(mode='EDIT', toggle=False)
    bpy.ops.object.hook_add_newob()
    bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

    control_hook = bpy.context.selected_objects[0]
    bpy.context.scene.objects.active = control_hook
    curve['rocket_hook'] = control_hook.name
    control_hook['parent_curve'] = curve.name

    ## Animate the hook
    bpy.ops.anim.change_frame(frame=start_frame)
    control_hook.location = [0.0, 1.0, 0.0]
    bpy.ops.anim.keyframe_insert(type=-1, confirm_success=False)
    bpy.ops.anim.change_frame(frame=end_frame)
    control_hook.location = [3.0, 1.0, 0.0]
    bpy.ops.anim.keyframe_insert(type=-1, confirm_success=False)

    # Figure out which point we want our "explosion" on
    ## We'll want to base the explosion coordinates on the 
    explosion_coord = (
        curve.data.splines[0].bezier_points[-1].co
        + curve.location)
    explosion_coord = explosion_coord + Mathutils.Vector([0, 0, 1])

    ## Create a circle and attach at that point
    bpy.ops.mesh.primitive_circle_add(
        vertices=32, fill=True,
        radius=curve.get('explosion_radius', 1),
        layer=[True] + [False] * 31,
        location=explosion_coord)

    explosion = bpy.context.scene.objects.active
    curve['explosion_circle'] = explosion.name
    explosion['parent_curve'] = curve.name

    explosion.data.add_material(curve.data.materials[0])

    ## Animate that circle.  The order is:
    ##  - Scaled down point
    ##  - Squashed point
    ##  - Final point
    ## ... but for laziness sake we will do it in reverse :)
    bpy.ops.anim.change_frame(frame=end_frame + 8)
    bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)
    bpy.ops.anim.change_frame(frame=end_frame + 6)
    bpy.ops.transform.resize(value=(1.1, 0.9, 1.1))
    bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)
    bpy.ops.anim.change_frame(frame=end_frame)
    bpy.ops.transform.resize(value=(0.0, 0.0, 0.0))
    bpy.ops.anim.keyframe_insert(type=-3, confirm_success=False)

    new_taper_curve.layers = NUKE_MACHINERY_LAYERS
    new_bevel.layers = NUKE_MACHINERY_LAYERS
    control_hook.layers = NUKE_MACHINERY_LAYERS
    explosion.layers = NUKE_EXPLOSION_LAYERS


class NukifyCurve(bpy.types.Operator):
    bl_idname = "patanim.nukify_curves"
    bl_label = "Nukify selected curves"

    def execute(self, context):
        curves = [
            obj for obj in bpy.context.selected_objects
            if obj.type == 'CURVE']

        for curve in curves:
            nukify_curve(curve)

        return {'FINISHED'}


bpy.types.register(NukifyCurve)
