import random
import bpy

def lazy_deselect_objects():
    if bpy.context.selected_objects:
        for obj in bpy.context.selected_objects:
            obj.selected = False


def _make_safe(decorator, original):
    """
    Copy the function data from the old function to the decorator.
    """
    decorator.__name__ = original.__name__
    decorator.__dict__ = original.__dict__
    decorator.__doc__ = original.__doc__
    return decorator


def opify_save_excursion(func):
    decorated_func = save_excursion(func)
    return lambda self, context: decorated_func(self, context)


def save_excursion(func):
    """
    Save excursion, kinda like in emacs lisp... we reset things to
    their previous state.  Works as long as things don't leave the
    scene or are deleted...
    """
    def new_func(*args, **kwargs):
        # get the original stuff
        orig_visible_layers = list(bpy.context.scene.visible_layers)
        orig_cursor_location = bpy.context.scene.cursor_location.copy()
        orig_active = bpy.context.scene.objects.active
        orig_selected = bpy.context.selected_objects
        orig_frame = bpy.context.scene.current_frame

        # call the method
        result = func(*args, **kwargs)

        # set everything back
        for obj in bpy.context.selected_objects or []:
            obj.selected = False

        for obj in orig_selected or []:
            if obj.name:
                obj.selected = True

        if orig_active.name:
            bpy.context.scene.objects.active = orig_active

        bpy.context.scene.cursor_location = orig_cursor_location
        bpy.context.scene.visible_layers = orig_visible_layers
        bpy.context.scene.set_frame(orig_frame)

        return result

    return _make_safe(new_func, func)


@save_excursion
def possibly_delete_object_by_id(obj_id):
    if obj_id not in bpy.context.scene.objects:
        return False

    lazy_deselect_objects()
    bpy.context.scene.objects[obj_id].selected = True

    bpy.ops.object.delete()

    return True
